package be.iminds.ilabt.jfed.experimenter_gui.config.util;

import be.iminds.ilabt.jfed.experimenter_gui.config.JFedGuiConfig;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Resource;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.ResourceClass;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Testbed;
import be.iminds.ilabt.jfed.lowlevel.testbed_info.TestbedInfoSource;
import be.iminds.ilabt.jfed.lowlevel.authority.finder.AuthorityFinder;
import be.iminds.ilabt.jfed.rspec.generator.StitchingTestRSpecGenerator;
import be.iminds.ilabt.jfed.rspec.model.*;
import be.iminds.ilabt.util.jsonld.impl.PrimaryIdObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * GuiConfigRSpecGenerator
 */
public class GuiConfigStitchingTestRSpecGenerator implements StitchingTestRSpecGenerator {
    private static final Logger LOG = LoggerFactory.getLogger(GuiConfigStitchingTestRSpecGenerator.class);

    @Nonnull private final JFedGuiConfig jFedGuiConfig;
    @Nonnull private final AuthorityFinder authorityFinder;
    @Nonnull private final TestbedInfoSource testbedInfoSource;

    private List<Server> servers = null;
    private List<String> nodeNames = Collections.emptyList();
    private String nodeNamePrefix = "n";

    public GuiConfigStitchingTestRSpecGenerator(@Nonnull JFedGuiConfig jFedGuiConfig,
                                                @Nonnull AuthorityFinder authorityFinder,
                                                @Nonnull TestbedInfoSource testbedInfoSource) {
        this.jFedGuiConfig = jFedGuiConfig;
        this.authorityFinder = authorityFinder;
        this.testbedInfoSource = testbedInfoSource;
    }

    @Override
    public void setAms(@Nonnull List<String> ams) {
        assert !ams.isEmpty();
        servers = new ArrayList<>();
        for (String am: ams) {
            Server server = null;
            if (am.startsWith("urn:")) {
                server = authorityFinder.findByAnyUrn(am, AuthorityFinder.Purpose.REQUEST_RSPEC);
            } else {
                if (am.matches("[0-9]*")) {
                    server = testbedInfoSource.getServerById(Integer.parseInt(am));
                } else {
                    Testbed testbed = testbedInfoSource.getTestbedById(am);
                    server = testbed.getDefaultServerId() == null ?
                            null :
                            testbedInfoSource.getServerById(testbed.getDefaultServerId());
                }
            }
            if (server == null)
                throw new RuntimeException("Did not find Server for AM \"" + am + "\" " +
                        "(Only URN, Testbed ID (=string) and Server ID (=int) are supported)");
            servers.add(server);
        }
        assert !servers.isEmpty();
        assert servers.size() == ams.size();
    }

    @Override
    public void setNodeNames(@Nonnull List<String> nodeNames) {
        this.nodeNames = nodeNames;
    }

    @Override
    public void setNodePrefix(@Nullable String prefix) {
        this.nodeNamePrefix = prefix;
    }

    @Nonnull
    @Override
    public String generateRequest() {
        if (servers == null || servers.isEmpty())
            throw new IllegalStateException("generateRequest requires setAMs(...) to set the servers");

        RspecFactory factory = RspecFactoryFactory.getRspecFactoryInstance(ModelRspecType.BASIC);
        ModelRspec model = factory.createModelRspec("request");

        int nodeCount = servers.size();

        for (int i = 0; i < nodeCount; i++) {
            Server server = servers.get(i);
            List<ResourceClass> resourceClassList = jFedGuiConfig.findAllPossibleResourceClasses(server);
            ResourceClass chosenResourceClass = null;
            for (ResourceClass resourceClass : resourceClassList) {
                //highest weight is preferred
                if (chosenResourceClass == null || resourceClass.getWeightOrZero() > chosenResourceClass.getWeightOrZero()) {
                    chosenResourceClass = resourceClass;
                }
            }
            if (chosenResourceClass == null) {
                throw new RuntimeException("There is no resourceClass for server "+server.getId()+" that can be used.");
            }
            LOG.debug("Chose ResourceClass "+chosenResourceClass.getId()+" from server ResourceClasses: "+
                    resourceClassList.stream()
                            .map(PrimaryIdObject::getId)
                            .collect(Collectors.joining(", "))
            );


            if (chosenResourceClass.getId().equals(JFedGuiConfig.RESOURCE_CLASS_ID_OTHER_SERVERS)) {
                throw new RuntimeException("There is no resource for server "+server.getId()+" " +
                        "(\""+server.getName()+"\"), cannot generate RSpec.");
            }

            Resource chosenResource = null;
            for (Resource aResource: chosenResourceClass.getResources()) {
                if (aResource.getServerId() != null && aResource.getServerId().equals(server.getId())) {
                    LOG.debug("Found Resource for server: "+chosenResource);
                    chosenResource = aResource;
                }
            }

            if (chosenResource == null) {
                chosenResource = jFedGuiConfig.getDefaultResource(chosenResourceClass);
                LOG.debug("Chose default Resource: "+chosenResource+"  because none matched server out of "+
                        chosenResourceClass.getResources().stream()
                                .map(PrimaryIdObject::getId)
                                .map(Object::toString)
                                .collect(Collectors.joining(", "))
                );
                throw new RuntimeException("No Resource found for server "+server.getId()+"");
            }
            LOG.debug("Chose Resource: "+chosenResource);

            if (chosenResource.getRspecElementName() != null &&
                    !chosenResource.getRspecElementName().equalsIgnoreCase("node")) {
                throw new RuntimeException("Resource for server "+server.getId()+" found, " +
                        "but has unsupported RSpec element name: " +
                        "\""+chosenResource.getRspecElementName()+"\"");
            }



            RspecNode node = factory.createNode(model);
            node.setComponentManagerId(server.getDefaultComponentManagerUrn());
            node.setExclusive(chosenResource.getDefaultExclusive());


            if (i < nodeNames.size()) {
                node.setClientId(nodeNames.get(i));
            } else {
                node.setClientId(nodeNamePrefix + i);
            }

            node.setSliverTypeName(chosenResource.getSliverType());

            if (chosenResource.getHardwareType() != null) {
                node.getHardwareTypes().add(new HardwareType(chosenResource.getHardwareType()));
            }
            if (chosenResource.getDiskImage() != null) {
                node.setSliverTypeDiskImage(chosenResource.getDiskImage());
            }

            model.addNode(node);
        }

        int j = 0;
        List<RspecNode> otherNodes = new ArrayList<>(model.getNodes());
        for (RspecNode node: model.getNodes()) {
            assert node.getClientId() != null;
            boolean removed = otherNodes.remove(node);
            assert removed;
            for (RspecNode otherNode: otherNodes) {
                assert otherNode.getClientId() != null;
                assert node != otherNode;
                assert !node.getClientId().equals(otherNode.getClientId());

                RspecLink link = factory.createLinkWithClientId(model, "link"+(j), null/*"lan"*/);
                model.addLink(link);

                RspecInterface iface = factory.createInterface(node, link, node.getClientId() + ":if"+node.getInterfaces().size());
                iface.getIpAddresses().add(new RspecInterface.IpAddress("192.168."+(j+2)+".1", "255.255.255.0", "ipv4"));
                RspecInterface otherIface = factory.createInterface(otherNode, link, otherNode.getClientId() + ":if"+otherNode.getInterfaces().size());
                otherIface.getIpAddresses().add(new RspecInterface.IpAddress("192.168."+(j+2)+".2", "255.255.255.0", "ipv4"));

                //set default capacity for stitching to 10Mbps
                link.getLinkSetting(iface, otherIface).setCapacity_Kbps(10_000L);
                link.getLinkSetting(otherIface, iface).setCapacity_Kbps(10_000L);

                //apparently not needed
//                link.getComponentManagerUrns().add(node.getComponentManagerId());
//                if (! node.getComponentManagerId().equals(otherNode.getComponentManagerId())) {
//                    //stitching link
//                    link.getComponentManagerUrns().add(otherNode.getComponentManagerId());
//                }

                j++;
            }
        }

        return model.toGeni3Rspec();
    }
}
